@@ -23,6 +23,8 @@
 
 #include <gst/gst.h>
 #include <gst/base/gstadapter.h>
+#include <gst/base/gstflowcombiner.h>
+#include "gstisoff.h"
 
 G_BEGIN_DECLS
 
@@ -64,6 +66,8 @@ struct _GstQTDemux {
   gint     n_audio_streams;
   gint     n_sub_streams;
 
+  GstFlowCombiner *flowcombiner;
+
   gboolean have_group_id;
   guint group_id;
 
@@ -73,11 +77,10 @@ struct _GstQTDemux {
   GNode *moov_node_compressed;
 
   guint32 timescale;
-  guint64 duration;
+  GstClockTime duration;
 
   gboolean fragmented;
-  /* offset of the mfra atom */
-  guint64 mfra_offset;
+  gboolean fragmented_seek_pending;
   guint64 moof_offset;
 
   gint state;
@@ -106,6 +109,7 @@ struct _GstQTDemux {
   guint64 mdatoffset;
   guint64 first_mdat;
   gboolean got_moov;
+  guint64 last_moov_offset;
   guint header_size;
 
   GstTagList *tag_list;
@@ -113,7 +117,10 @@ struct _GstQTDemux {
   /* configured playback region */
   GstSegment segment;
   GstEvent *pending_newsegment;
-  gboolean upstream_newsegment;
+  gboolean upstream_format_is_time; /* qtdemux received upstream
+                                     * newsegment in TIME format which likely
+                                     * means that upstream is driving the pipeline
+                                     * (adaptive demuxers / dlna) */
   gint64 seek_offset;
   gint64 push_seek_start;
   gint64 push_seek_stop;
@@ -131,12 +138,21 @@ struct _GstQTDemux {
    * upstream provides it at the caps */
   GstCaps *media_caps;
   gboolean exposed;
-  gboolean mss_mode; /* flag to indicate that we're working with a smoothstreaming fragment */
+  gboolean mss_mode; /* flag to indicate that we're working with a smoothstreaming fragment
+                      * Mss doesn't have 'moov' or any information about the streams format,
+                      * requiring qtdemux to expose and create the streams */
   guint64 fragment_start;
+  guint64 fragment_start_offset;
     
   gint64 chapters_track_id;
 
-  GstClockTime min_elst_offset;
+  /* protection support */
+  GPtrArray *protection_system_ids; /* Holds identifiers of all content protection systems for all tracks */
+  GQueue protection_event_queue; /* holds copy of upstream protection events */
+  guint64 cenc_aux_info_offset;
+  guint8 *cenc_aux_info_sizes;
+  guint32 cenc_aux_sample_count;
+
 };
 
 struct _GstQTDemuxClass {
